#! /bin/bash

trap "echo '*** BUILD FAILED! ***' 1>&2" ERR
set -ex -o pipefail
umask 022
export PATH='/bin:/usr/bin:/usr/local/bin:~/diet-pc/tools'
export CFLAGS=${CFLAGS:--Os}
export CXXFLAGS=${CXXFLAGS:--Os}

rebuild () {
    tar xjf libfontenc-1.1.1.tar.bz2
    cd libfontenc-1.1.1
    ./configure --prefix=/usr --sysconfdir=/etc/X11 --localstatedir=/var \
	    --datadir=/usr/lib --enable-static
    make
    cd ..

    tar xjf libXfont-1.3.4.tar.bz2
    cd libXfont-1.3.4
    ./configure --prefix=/usr --sysconfdir=/etc/X11 --localstatedir=/var \
	    --datadir=/usr/lib --enable-static --disable-freetype \
	    --disable-type1 --disable-speedo --disable-bdfformat \
	    --disable-snfformat --disable-builtins
    make
    cd ..

    tar xjf MesaLib-6.5.2.tar.bz2

    # FIXME: lib/X11/xkb/rules/base[+xorg]?

    tar xjf xorg-server-1.2.0.tar.bz2
    cd xorg-server-1.2.0
    bunzip2 -c ../xorg-xserver-1.2.0-xcmisc.diff.bz2 | patch -p1 -b
    bunzip2 -c ../xorg-xserver-1.2-multiple-overflows-v2.diff.bz2 | patch -p1 -b
    bunzip2 -c ../xorg-server-1.2.0-xnest-fullscreen.patch.bz2 | patch -p1 -b
    bunzip2 -c ../xorg-server-1.2.0-ephyr-fullscreen.patch.bz2 | patch -p1 -b
    mkdir -p ../xserver-kdrive-fbdev/lib/xserver ../xephyr/lib/xserver \
	    ../xnest/lib/xserver ../xserver-xorg/lib/xserver
    cp Xext/SecurityPolicy ../xserver-kdrive-fbdev/lib/xserver/
    cp Xext/SecurityPolicy ../xephyr/lib/xserver/
    cp Xext/SecurityPolicy ../xnest/lib/xserver/
    cp Xext/SecurityPolicy ../xserver-xorg/lib/xserver/
    ./configure --prefix=/usr --sysconfdir=/etc/X11 --localstatedir=/var \
	    --datadir=/usr/lib --disable-static \
	    --with-dri-driver-path=/usr/lib/xorg/modules/dri \
	    --disable-xorgcfg --disable-glx --disable-xvfb --disable-xprint \
	    --disable-dmx --disable-xorg --enable-xnest --enable-kdrive \
	    --with-mesa-source=`realpath ../Mesa-6.5.2`
    ed -s hw/kdrive/fbdev/Makefile >/dev/null <<EOF
/^Xfbdev_LDADD =/
/-lXfont -lXau -lfontenc/
s@-lXfont -lXau -lfontenc@../../../../libfontenc-1.1.1/src/.libs/libfontenc.a -lz ../../../../libXfont-1.3.4/src/.libs/libXfont.a -lXau@
w
q
EOF
    ed -s hw/kdrive/ephyr/Makefile >/dev/null <<EOF
/^Xephyr_LDADD =/
/-lXfont -lXau -lfontenc/
s@-lXfont -lXau -lXdmcp   -lXfont -lXau -lfontenc@../../../../libfontenc-1.1.1/src/.libs/libfontenc.a -lz ../../../../libXfont-1.3.4/src/.libs/libXfont.a -lXau@
w
q
EOF
    ed -s hw/xnest/Xnest >/dev/null <<EOF
/^XNESTMODULES_LIBS =/
s@-lXfont@../../../libfontenc-1.1.1/src/.libs/libfontenc.a -lz -lm ../../../libXfont-1.3.4/src/.libs/libXfont.a@
w
q
EOF
# FIXME: vesa? (or lack thereof, on x86_64)
    make
    strip -R .note -R .comment hw/kdrive/fbdev/Xfbdev hw/kdrive/ephyr/Xephyr \
	    hw/xnest/Xnest
    mkdir -p ../xserver-kdrive-fbdev/bin ../xephyr/bin ../xnest/bin
    cp hw/kdrive/fbdev/Xfbdev ../xserver-kdrive-fbdev/bin/
    cp hw/kdrive/ephyr/Xephyr ../xephyr/bin/
    cp hw/xnest/Xnest ../xnest/bin/
    ./configure --prefix=/usr --sysconfdir=/etc/X11 --localstatedir=/var \
	    --datadir=/usr/lib --disable-static \
	    --with-dri-driver-path=/usr/lib/xorg/modules/dri \
	    --disable-xorgcfg --with-mesa-source=`realpath ../Mesa-6.5.2` \
	    --disable-xvfb --disable-xprint --disable-dmx --disable-xnest \
	    --disable-kdrive --enable-xorg
    ed -s hw/xfree86/Makefile >/dev/null <<EOF
/^Xorg_LDADD =/
/-lXfont -lXau -lfontenc/
s@-lXfont -lXau -lfontenc@../../../libfontenc-1.1.1/src/.libs/libfontenc.a -lz ../../../libXfont-1.3.4/src/.libs/libXfont.a -lXau@
w
q
EOF
    make
    cd hw/xfree86
    strip -R .note -R .comment Xorg */.libs/*.so \
	    dixmods/extmod/.libs/libextmod.so
    cp Xorg ../../../xserver-xorg/bin/
    mkdir -p ../../../xserver-xorg/lib/xorg/modules/{extensions,linux}
    cp ddc/.libs/libddc.so ../../../xserver-xorg/lib/xorg/modules/
    cp dixmods/.libs/lib{fb,shadow}.so int10/.libs/libint10.so \
	    vbe/.libs/libvbe.so ../../../xserver-xorg/lib/xorg/modules/
    cp dixmods/extmod/.libs/libextmod.so dixmods/.libs/libdbe.so \
	    ../../../xserver-xorg/lib/xorg/modules/extensions/
    cp fbdevhw/.libs/libfbdevhw.so \
	    ../../../xserver-xorg/lib/xorg/modules/linux/
    cp i2c/.libs/libi2c.so ../../../xserver-xorg/lib/xorg/modules/
    cp scanpci/.libs/libpcidata.so ../../../xserver-xorg/lib/xorg/modules/
    mkdir -p ../../../xserver-xorg-dri/lib/xorg/modules/extensions
    cp dixmods/.libs/libglx.so dri/.libs/libdri.so \
	    ../../../xserver-xorg-dri/lib/xorg/modules/extensions/
    mkdir -p ../../../xserver-xorg-accel/lib/xorg/modules
    cp ramdac/.libs/libramdac.so shadowfb/.libs/libshadowfb.so \
	    vgahw/.libs/libvgahw.so xaa/.libs/libxaa.so \
	    ../../../xserver-xorg-accel/lib/xorg/modules/
    mkdir -p ../../../xserver-xorg-softglx/lib/xorg/modules/extensions
    cp dixmods/.libs/libGLcore.so \
	    ../../../xserver-xorg-softglx/lib/xorg/modules/extensions/
    cd ../../..
    #rm -rf xorg-server-1.2.0

    mkdir -p xserver-xorg/lib/xorg/modules/input
    tar xjf xf86-input-keyboard-1.1.1.tar.bz2
    cd xf86-input-keyboard-1.1.1
    ./configure --prefix=/usr --sysconfdir=/etc/X11 --localstatedir=/var \
	    --datadir=/usr/lib --disable-static
    make
    strip -R .note -R .comment src/.libs/keyboard_drv.so
    cp src/.libs/keyboard_drv.so ../xserver-xorg/lib/xorg/modules/input/
    cd ..
    rm -rf xf86-input-keyboard-1.1.1
    tar xjf xf86-input-mouse-1.1.2.tar.bz2
    cd xf86-input-mouse-1.1.2
    ./configure --prefix=/usr --sysconfdir=/etc/X11 --localstatedir=/var \
	    --datadir=/usr/lib --disable-static
    make
    strip -R .note -R .comment src/.libs/mouse_drv.so
    cp src/.libs/mouse_drv.so ../xserver-xorg/lib/xorg/modules/input/
    cd ..
    rm -rf xf86-input-mouse-1.1.2

    mkdir -p xserver-xorg/lib/xorg/modules/drivers
    tar xjf xf86-video-fbdev-0.4.2.tar.bz2
    cd xf86-video-fbdev-0.4.2
    ./configure --prefix=/usr --sysconfdir=/etc/X11 --localstatedir=/var \
	    --datadir=/usr/lib --disable-static
    make
    strip -R .note -R .comment src/.libs/fbdev_drv.so
    cp src/.libs/fbdev_drv.so ../xserver-xorg/lib/xorg/modules/drivers/
    cd ..
    rm -rf xf86-video-fbdev-0.4.2
    tar xjf xf86-video-vesa-1.3.0.tar.bz2
    cd xf86-video-vesa-1.3.0
    ./configure --prefix=/usr --sysconfdir=/etc/X11 --localstatedir=/var \
	    --datadir=/usr/lib --disable-static
    make
    strip -R .note -R .comment src/.libs/vesa_drv.so
    cp src/.libs/vesa_drv.so ../xserver-xorg/lib/xorg/modules/drivers/
    cd ..
    rm -rf xf86-video-vesa-1.3.0
}

for PKG in xephyr xnest xserver-xorg; do
   mkdir -p $PKG/lib/X11/xkb/rules
   ln -sf base $PKG/lib/X11/xkb/rules/xorg
done

while read FN; do
    eval "[ -e $FN ] || rebuild"
done < MANIFEST

ARCH=`ipkg -V 0 print_architecture | tail -n 1 | cut -f2 -d' '`
for CTRLFILE in */CONTROL/control; do
    sed -i -e "s/__ARCH__/$ARCH/g" $CTRLFILE
    sudo ipkg-build -o root -g root `dirname $CTRLFILE`/..
done
